#include < amxmodx >
#include < amxmisc >
#include < celltravtrie >

static PLUGIN[ ]         =         "End Round Sounds with Musical Genre"
static VERSION[ ]        =         "1.0.1"
static AUTHOR[ ]         =         "guipatinador"

#define MAX_GENRES              8
#define MAX_LENGHT_SOUND        64
#define MAX_LENGHT_GENRE        24
#define MAX_PLAYERS             32

static FILE_NAME[ ]      =        "ers_genre.ini"
static DICTIONARY[ ]     =        "ers_genre.txt"

new TravTrie:g_tERS_Genres
new TravTrie:g_tERS_Sounds[ MAX_GENRES ]
new TravTrie:g_tERS_Is_Mp3[ MAX_GENRES ]

enum ( += 1 ) {
	INVALID = 0,
	MP3,
	WAV
}

new g_pcvarAdds
new g_pcvarTimeBetWeenAds
new g_pcvarColoredMessages

new g_iTotalGenreCount
new g_menu

new music_type[ MAX_PLAYERS + 1 ]
new bool:is_first_round[ MAX_PLAYERS + 1 ]

public plugin_precache( )
{
	register_dictionary( DICTIONARY )
	
	new szFilePath[ 128 ]
	get_configsdir( szFilePath, charsmax( szFilePath ) )
	format( szFilePath, charsmax( szFilePath ), "%s/%s", szFilePath, FILE_NAME )
	
	new f = fopen( szFilePath, "rt" )
	
	if( !f )
	{
		new szMessage[ 256 ]
		formatex( szMessage, charsmax( szMessage ), "%L", LANG_SERVER, "ERROR_OPEN_FILE", szFilePath )
		set_fail_state( szMessage )
	}
	
	new szData[ MAX_LENGHT_SOUND ]
	new iSounds
	
	g_tERS_Genres = TravTrieCreate( )
	
	while( !feof( f ) )
	{
		fgets( f, szData, charsmax( szData ) )
		
		replace_all( szData, charsmax( szData ), "^n", "" )
		
		if( szData[ 0 ] == '[' )
		{				
			replace_all( szData, charsmax( szData ), "[", "" )
			replace_all( szData, charsmax( szData ), "]", "" )
			trim( szData )		
			
			g_tERS_Sounds[ g_iTotalGenreCount ] = TravTrieCreate( )
			g_tERS_Is_Mp3[ g_iTotalGenreCount ] = TravTrieCreate( )
			TravTrieSetStringEx( g_tERS_Genres, g_iTotalGenreCount, szData )
			
			g_iTotalGenreCount++
			iSounds = 0
		}
		
		else if( szData[ 0 ] == '"' )
		{
			if( !g_iTotalGenreCount )
			{
				continue
			}
			
			remove_quotes( szData )
			trim( szData )
			
			if( !equal( szData, "sound/", 6 ) )
			{
				format( szData, charsmax( szData ), "sound/%s", szData )
			}
			
			if( file_exists( szData ) )
			{				
				switch( file_extension( szData ) )
				{
					case MP3:
					{
						TravTrieSetStringEx( g_tERS_Sounds[ g_iTotalGenreCount - 1 ], iSounds, szData )
						TravTrieSetCellEx( g_tERS_Is_Mp3[ g_iTotalGenreCount - 1 ], iSounds, true )
						iSounds++
						
						precache_generic( szData )
					}
					
					case WAV:
					{
						replace_all( szData, charsmax( szData ), "sound/", "" )
						
						TravTrieSetStringEx( g_tERS_Sounds[ g_iTotalGenreCount - 1 ], iSounds, szData )
						TravTrieSetCellEx( g_tERS_Is_Mp3[ g_iTotalGenreCount - 1 ], iSounds, false )
						iSounds++
						
						precache_sound( szData )	
					}
					
					case INVALID:
					{
						log_amx( "%L", LANG_SERVER, "UNABLE_TO_PRECACHE", szData )
					}
				}
			}
			
			else
			{
				log_amx( "%L", LANG_SERVER, "UNABLE_TO_PRECACHE", szData )
			}
		}
	}
	
	fclose( f )
}

public plugin_init( ) 
{
	register_plugin( PLUGIN, VERSION, AUTHOR )
	
	register_clcmd( "say /roundsound", "ChangeGenre" )
	register_clcmd( "say /roundsounds", "ChangeGenre" )
	register_clcmd( "say_team /roundsound", "ChangeGenre" )
	register_clcmd( "say_team /roundsounds", "ChangeGenre" )
	
	g_pcvarAdds = register_cvar( "ers_ads", "1" )
	g_pcvarTimeBetWeenAds =	register_cvar( "ers_time_between_ads", "300" )
	g_pcvarColoredMessages = register_cvar( "ers_colored_messages", "1" )
	
	register_logevent( "EventStartRound", 2, "1=Round_Start" )  
	register_logevent( "EventEndRound", 2, "1=Round_End" )
	
	if( get_pcvar_num( g_pcvarAdds ) )
	{
		set_task( get_pcvar_float( g_pcvarTimeBetWeenAds ), "Ads", _, _, _, "b" )
	}
}

public plugin_cfg( )
{
	if( !g_iTotalGenreCount )
	{
		new szMessage[ 256 ]
		formatex( szMessage, charsmax( szMessage ), "%L", LANG_SERVER, "NO_MUSICAL_GENRES" )
		set_fail_state( szMessage )
	}
	
	new szMenuTitle[ 64 ]
	formatex( szMenuTitle, charsmax( szMenuTitle ), "%L", LANG_SERVER, "FAVORITE_GENRE" )
	
	g_menu = menu_create( szMenuTitle, "MenuHandler" )
	
	new i
	new szGenre[ MAX_LENGHT_GENRE ]
	new szFormatedGenre[ MAX_LENGHT_GENRE + 2 ]
	new szNum[ 4 ]
	
	for( i = 0; i < g_iTotalGenreCount; i++ )
	{
		TravTrieGetStringEx( g_tERS_Genres, i, szGenre, charsmax( szGenre ) )
		formatex( szFormatedGenre, charsmax( szFormatedGenre ), "\w%s", szGenre )
		num_to_str( i, szNum, charsmax( szNum ) )
		
		menu_additem( g_menu, szFormatedGenre )
	}
	
	new iKeyOfNoSounds = i + 1
	num_to_str( iKeyOfNoSounds, szNum, charsmax( szNum ) )
	
	new szWithoutSounds[ 64 ]
	formatex( szWithoutSounds, charsmax( szWithoutSounds ), "%L", LANG_SERVER, "WITHOUT_SOUNDS" )
	
	menu_additem( g_menu, szWithoutSounds )	
	
	menu_setprop( g_menu, MPROP_EXIT, MEXIT_NEVER )
}

public client_connect( id )
{
	is_first_round[ id ] = true
	music_type[ id ] = -1
}

public Ads(  )
{	
	if( get_pcvar_num( g_pcvarAdds ) )
	{
		new iColoredMessage = get_pcvar_num( g_pcvarColoredMessages )
		
		new Players[ 32 ]
		new iNum
		new i
		
		get_players( Players, iNum, "ch" )
		
		if( iColoredMessage )
		{
			for( --iNum; iNum >= 0; iNum-- )
			{
				i = Players[ iNum ]
				ClientPrintColor( i, "%L", i, "COLORED_ADDS" )
			}
		}
		
		else
		{
			for( --iNum; iNum >= 0; iNum-- )
			{
				i = Players[ iNum ]
				client_print( i, print_chat, "%L", i, "ADDS" )				
			}	
		}
	}	
}

public EventStartRound( )
{
	new Players[ 32 ]
	new iNum
	new i
	
	get_players( Players, iNum, "ch" )
	
	for( --iNum; iNum >= 0; iNum-- )
	{
		i = Players[ iNum ]
		
		if( is_user_alive( i ) && is_first_round[ i ] )
		{
			ChangeGenre( i )
		}
	}
}

public ChangeGenre( id )
{
	menu_display( id, g_menu, 0 )
}

public MenuHandler( id, g_menu, item )
{
	if( item == g_iTotalGenreCount )
	{	
		music_type[ id ] = -1
		
		if( get_pcvar_num( g_pcvarColoredMessages ) )
		{
			ClientPrintColor( id, "%L", id, "COLORED_SOUNDS_DISABLED" )
		}
		
		else
		{
			client_print( id, print_chat, "%L", id, "SOUNDS_DISABLED" )	
		}
	}
	
	else
	{
		music_type[ id ] = item
		
		new szGenre[ MAX_LENGHT_GENRE ]
		TravTrieGetStringEx( g_tERS_Genres, music_type[ id ], szGenre, charsmax( szGenre ) )
		
		if( get_pcvar_num( g_pcvarColoredMessages ) )
		{
			ClientPrintColor( id, "%L", id, "COLORED_SOUNDS_ENABLED", szGenre )
		}
		
		else
		{
			client_print( id, print_chat, "%L", id, "SOUNDS_ENABLED", szGenre )	
		}
	}	
	
	is_first_round[ id ] = false
	
	return PLUGIN_HANDLED
}

public EventEndRound( )
{
	new iRandomSound[ MAX_GENRES ]
	
	new j
	for( j = 0; j < g_iTotalGenreCount; j++ )
	{
		iRandomSound[ j ] = random( TravTrieSize( g_tERS_Sounds[ j ] ) )
	}
	
	new Players[ 32 ]
	new iNum
	new i
	
	new szSound[ MAX_LENGHT_SOUND ]
	new music_index
	new bool:is_mp3
	
	get_players( Players, iNum, "ch" )
	
	for( --iNum; iNum >= 0; iNum-- )
	{
		i = Players[ iNum ]
		music_index = music_type[ i ]
		
		if( music_index == -1 )
		{
			continue
		}
		
		TravTrieGetStringEx( g_tERS_Sounds[ music_index ], iRandomSound[ music_index ], szSound, charsmax( szSound ) )
		TravTrieGetCellEx( g_tERS_Is_Mp3[ music_index ], iRandomSound[ music_index ], is_mp3 )
		
		switch( is_mp3 )
		{
			case true:
			{
				client_cmd( i, "mp3 play %s", szSound )
			}
			
			case false:
			{
				client_cmd( i, "spk %s", szSound )
			}
		}	
	}
}

stock file_extension( szSound[ ] )
{
	new iLen = strlen( szSound )
	
	if( szSound[ iLen - 1 ] == '3' && ( szSound[ iLen - 2 ] == 'p' || szSound[ iLen - 2 ] == 'P' ) && ( szSound[ iLen - 3 ] == 'm' || szSound[ iLen - 3 ] == 'M' ) && szSound[ iLen - 4 ] == '.' )
	{
		return MP3
	}
	
	if( ( szSound[ iLen - 1 ] == 'v' || szSound[ iLen - 1 ] == 'V' ) && ( szSound[ iLen - 2 ] == 'a' || szSound[ iLen - 2 ] == 'A' ) && ( szSound[ iLen - 3 ] == 'w' || szSound[ iLen - 3 ] == 'W' ) && szSound[ iLen - 4 ] == '.' )
	{
		return WAV
	}
	
	return INVALID	
}

stock ClientPrintColor( id, String[ ], any:... )
{
	new szMsg[ 192 ]
	vformat( szMsg, charsmax( szMsg ), String, 3 )
	
	replace_all( szMsg, charsmax( szMsg ), "!n", "^1" )
	replace_all( szMsg, charsmax( szMsg ), "!t", "^3" )
	replace_all( szMsg, charsmax( szMsg ), "!g", "^4" )
	
	static msgSayText = 0
	static fake_user
	
	if( !msgSayText )
	{
		msgSayText = get_user_msgid( "SayText" )
		fake_user = get_maxplayers( ) + 1
	}
	
	message_begin( id ? MSG_ONE_UNRELIABLE : MSG_BROADCAST, msgSayText, _, id )
	write_byte( id ? id : fake_user )
	write_string( szMsg )
	message_end( )
}
/* AMXX-Studio Notes - DO NOT MODIFY BELOW HERE
*{\\ rtf1\\ ansi\\ deff0{\\ fonttbl{\\ f0\\ fnil Tahoma;}}\n\\ viewkind4\\ uc1\\ pard\\ lang2070\\ f0\\ fs16 \n\\ par }
*/
